/*  Copyright (c) January 2005 Jean Gressmann (jsg@rz.uni-potsdam.de)
 *
 *  This is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version. 
 * 
 *	This file is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this file; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef ARCH_COMMON_H
#define ARCH_COMMON_H

#include <iostream>

#define PT_ARCH_COMMON_INCLUDED

namespace PortableThreads 
{
	// Interger type definitions based on size
	namespace Private
	{
		template<unsigned>
		struct PTSizeToType_;

		template<>
		struct PTSizeToType_<1>
		{
			typedef int8 int_type;
			typedef uint8 uint_type;
		};

		template<>
		struct PTSizeToType_<2>
		{
			typedef int16 int_type;
			typedef uint16 uint_type;
		};

		template<>
		struct PTSizeToType_<4>
		{
			typedef int32 int_type;
			typedef uint32 uint_type;
		};

		template<>
		struct PTSizeToType_<8>
		{
			typedef int64 int_type;
			typedef uint64 uint_type;
		};
	}

	typedef Private::PTSizeToType_<sizeof(void*)>::int_type pt_pointer_type;
	typedef Private::PTSizeToType_<sizeof(long)>::int_type pt_int_type;
	typedef Private::PTSizeToType_<sizeof(unsigned long)>::uint_type pt_uint_type;

	namespace LockFree
	{
		namespace Private
		{
			class token_t 
			{ 
			public:	
				typedef ::PortableThreads::Private::PTSizeToType_<sizeof(void*)>::int_type int_t;
				explicit
				token_t(int_t value = 0, int_t count = 0)
					:	value_(value)
					,	count_(count)
				{}
				inline int_t value() const { return value_; }
				inline void value(int_t v) { value_ = v; }
				inline int_t count() const { return count_; }
				inline void count(int_t c) { count_ = c; }
			private:
				int_t value_, count_;
			};

			inline bool operator==(const token_t& lhs, const token_t& rhs)
			{
				return lhs.value() == rhs.value() && lhs.count() == rhs.count();
			}

			inline bool operator!=(const token_t& lhs, const token_t& rhs)
			{
				return !(lhs == rhs);
			}

			template<typename A, unsigned>
			class PT4BytePointerAllocator
			{
			public:
				typedef A allocator_type;
				typedef typename A::pointer pointer;
				typedef typename A::size_type size_type;
				explicit
				PT4BytePointerAllocator()
				{}
				explicit
				PT4BytePointerAllocator(allocator_type& a)
					:	allocator_(a)
				{}
				template<typename T>
				inline pointer allocate(size_type s, T* hint = 0)
				{
					return allocator_.allocate(s, hint);
				}
				inline void deallocate(pointer p, size_type s)
				{
					allocator_.deallocate(p, s);
				}
			private:
				allocator_type allocator_;
			};

			template<typename A>
			class PT4BytePointerAllocator<A, 8>
			{
			public:
				typedef A allocator_type;
				typedef typename A::pointer pointer;
				typedef typename A::size_type size_type;
				explicit
				PT4BytePointerAllocator()
				{}
				explicit
				PT4BytePointerAllocator(allocator_type& a)
					:	allocator_(a)
				{}
				template<typename T>
				inline pointer allocate(size_type s, T* hint = 0)
				{
					pointer p = allocator_.allocate(s, hint);					
					if(reinterpret_cast<uint64>(p) > static_cast<uint64>(0xffffffff))
					{
						std::cout << "too large: " << (void*)p << "\n";
						allocator_.deallocate(p, s);
						return 0;
					}
					return p;
				}
				inline void deallocate(pointer p, size_type s)
				{
					allocator_.deallocate(p, s);
				}
			private:
				allocator_type allocator_;
			};
		}
	}
}



#endif
